-- @author  Sven777b (ICQ 95176001)
-- @date  29/07/08

agro720 = {};

function agro720:new(configFile, positionX, offsetY, positionZ, rotationY, customMt)

    if agro720_mt == nil then
        agro720_mt = Class(agro720, Vehicle);
    end;

    local mt = customMt;
    if mt == nil then
        mt = agro720_mt;
    end;
    local instance = Vehicle:new(configFile, positionX, offsetY, positionZ, rotationY, mt);

    local xmlFile = loadXMLFile("TempConfig", configFile);

    instance.extra_lights = {};
    instance.extra_lights.rear = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.extra_lights.rear#index"));
    instance.extra_lights.front = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.extra_lights.front#index"));
    instance.extra_lights.frontL = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.extra_lights.front#src"));
    instance.extra_lights.brake = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.extra_lights.brake#index"));
    instance.interior = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.interior#index"));
    instance.interiorL = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.interior#indexlit"));
    instance.brakeForce = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.brakeForce"), 10);
    instance.rundumleuchtenAnz = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.rundumleuchten#count"),0);
    instance.rundumleuchtenAn = false;
    instance.rundumleuchten = {};
    for i=1, instance.rundumleuchtenAnz do
        local objname = string.format("vehicle.rundumleuchten.light" .. "%d",i);
        instance.rundumleuchten[i] = {};
        instance.rundumleuchten[i].rotNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, objname .. "#rotNode"));
        instance.rundumleuchten[i].light = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, objname .. "#light"));
        instance.rundumleuchten[i].source = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, objname .. "#lightsource"));
        instance.rundumleuchten[i].speed = Utils.getNoNil(getXMLInt(xmlFile,  objname .. "#rotSpeed"), 1)/1000;
        instance.rundumleuchten[i].emit = Utils.getNoNil(getXMLBool(xmlFile, objname .. "#emitLight"), true);
        if not instance.rundumleuchten[i].emit and instance.rundumleuchten[i].source ~= nil then
           setVisibility(instance.rundumleuchten[i].source, false);
        end;
    end;
    instance.numTAJ = getXMLInt(xmlFile, "vehicle.trailerAttacher#count");
    instance.TAJ = {};
    for i=1,instance.numTAJ do
      local obj = string.format("vehicle.trailerAttacher.joint".."%d",i);
      instance.TAJ[i] = {};
      instance.TAJ[i].index = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, obj.."#index"));
      instance.TAJ[i].domv = Utils.getNoNil(getXMLBool(xmlFile, obj.."#moveable"),false);
      if instance.TAJ[i].domv then
        instance.TAJ[i].minY = Utils.getNoNil(getXMLFloat(xmlFile, obj.."#minY"),0);
        instance.TAJ[i].maxY = Utils.getNoNil(getXMLFloat(xmlFile, obj.."#maxY"),0);
      end;
    end;
    instance.hydro = {};
    instance.hydro.F1 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.hydraulics.front#index1"));
    instance.hydro.F2 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.hydraulics.front#index2"));
    instance.hydro.R1 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.hydraulics.rear#index1"));
    instance.hydro.R2 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.hydraulics.rear#index2"));
    instance.motorBrakeForce = instance.motor.brakeForce;
    local brakeSound = getXMLString(xmlFile, "vehicle.brakeSound#file");
    if brakeSound ~= nil and brakeSound ~= "" then
        instance.brakeSound = createSample("brakeSound");
        loadSample(instance.brakeSound, brakeSound, false);
        instance.brakeSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.brakeSound#pitchOffset"), 0);
    end;

    instance.exhaustParticleLow = {};
    Utils.loadParticleSystem(xmlFile, instance.exhaustParticleLow, "vehicle.exhaustParticleLow", instance.rootNode, false);
    instance.exhaustParticleHigh = {};
    Utils.loadParticleSystem(xmlFile, instance.exhaustParticleHigh, "vehicle.exhaustParticleHigh", instance.rootNode, false);

    instance.trailerAttacherJoint = instance.TAJ[1].index;
    instance.trailerAttacherJointIndex = 0;
    instance.attachedTrailer = nil;
    instance.mirrors = {};
    instance.mirrors[1] = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.mirrors.mirror1#index"));
    instance.mirrors[2] = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.mirrors.mirror2#index"));


    instance.keys = {};
    local i=0;
    while true do
        local baseName = string.format("vehicle.keys.input(%d)", i);
        local inputName = getXMLString(xmlFile, baseName.. "#name");
        if inputName == nil then
            break;
        end;
        local inputKey = getXMLString(xmlFile, baseName.. "#key");
        if Input[inputKey] == nil then
            print("Error: invalid key '" .. inputKey .. "'  for input event '" .. inputName .. "'");
            break;
        end;
        instance.keys[inputName] = Input[inputKey];
        i = i+1;
    end;

    delete(xmlFile);

    instance.showhelp = false;
    instance.hudInfoPoxX = 0.012;
    instance.hudInfoPoxY = 0.1825;
    instance.hudInfoWidth = 0.425;
    instance.hudInfoHeight = 0.1625;
    instance.hudInfoOverlay = Overlay:new("hudInfoOverlay", "data/missions/hud_help_base.png", instance.hudInfoPoxX, instance.hudInfoPoxY, instance.hudInfoWidth, instance.hudInfoHeight);

    instance.licht1 = false;
    instance.licht2 = false;
    instance.brakeOn = false;
    instance.drawMirror = getVisibility(instance.mirrors[1])
    instance.rearlightActive = false;
    instance.revLimit = 100;
    instance.maxRpm3 = instance.motor.maxRpm[3];
    instance.activeTAJ = 0;

    return instance;
end;

function agro720:keyEvent(unicode, sym, modifier, isDown)
    agro720:superClass().keyEvent(self, unicode, sym, modifier, isDown);

    if isDown and sym == self.keys.rearlight then
      self.rearlightActive = not self.rearlightActive;
      setVisibility(self.extra_lights.rear, self.rearlightActive);
    end;

    if isDown and sym == self.keys.rundumleuchte then
      self.rundumleuchtenAn = not self.rundumleuchtenAn;
      for i=1, self.rundumleuchtenAnz do
        setVisibility(self.rundumleuchten[i].light, self.rundumleuchtenAn);
      end;
    end;
    
    if isDown and sym == self.keys.spiegel then
    	self.drawMirror = not self.drawMirror;
    	setVisibility(self.mirrors[1], self.drawMirror);
    	setVisibility(self.mirrors[2], self.drawMirror);
    end;
    
    if isDown and sym == self.keys.hilfe then
      if self.hudInfoOverlay ~= nil then
      	self.showhelp = not self.showhelp;
      end;
    end;

end;

function agro720:update(dt)
    if self.isEntered then

        if Input.isKeyPressed(self.keys.rev_up) and self.revLimit < 100 then
          self.revLimit = self.revLimit + 1;
        elseif Input.isKeyPressed(self.keys.rev_down) and self.revLimit > 10 then
          self.revLimit = self.revLimit - 1;
        end;
        self.motor.maxRpm[3] = string.format("%d",(self.revLimit * self.maxRpm3 / 100));

        local brake = false;
        if self.motor.brakeForce == self.brakeForce and getInputAxis(Input.AXIS_Y) ~= 0 then
          brake = true;
          self.motor.maxRpm[3] = 0;
        end;
        if (self.movingDirection*self.lastSpeed*(-getInputAxis(Input.AXIS_Y))) < -0.001 then
           brake = true;
           self.brakeOn = true;
           self.motor.brakeForce = self.brakeForce;
        elseif not brake then
           self.motor.brakeForce = self.motorBrakeForce;
        end;
        setVisibility(self.extra_lights.brake, brake);
        if self.brakeOn and not brake then
            if self.brakeSound ~= nil then
                setSamplePitch(self.brakeSound, self.brakeSoundPitchOffset);
                playSample(self.brakeSound, 1, 1, 0);
            end;
            self.brakeOn = false;
        end;
        if getInputAxis(Input.AXIS_Y) ~= 0 and not brake then
        		Utils.setEmittingState(self.exhaustParticleLow, false);
        		Utils.setEmittingState(self.exhaustParticleHigh, true);
        else
        		Utils.setEmittingState(self.exhaustParticleLow, true);
        		Utils.setEmittingState(self.exhaustParticleHigh, false);
        end;

        if self.attachedTrailer == nil and self.numTAJ > 0 then
            local nearestTrailer = nil;
            local nearestDistance = 0.4;
            for i=1,self.numTAJ do
                local px, py, pz = getWorldTranslation(self.TAJ[i].index);
                for t=1, table.getn(g_currentMission.trailers) do
                    local vx, vy, vz = getWorldTranslation(g_currentMission.trailers[t].attacherJoint);
                    local distance = Utils.vector2Length(px-vx, pz-vz);
                    if distance < nearestDistance then
                        self.trailerAttacherJoint = self.TAJ[i].index;
                        self.activeTAJ = i;
                        nearestDistance = distance;
                    end;
                end;
            end;
        end;

        if Input.isKeyPressed(self.keys.moveAK_up) or Input.isKeyPressed(self.keys.moveAK_dn) then
            setJointFrame(self.trailerAttacherJointIndex,0,self.trailerAttacherJoint);
            local ymod = 0;
            if Input.isKeyPressed(self.keys.moveAK_up) then ymod = 0.01; else ymod = -0.01; end;
            for i=1,self.numTAJ do
                if self.TAJ[i].domv then
                    local x,y,z = getTranslation(self.TAJ[i].index);
                    y = y + ymod;
                    if y > self.TAJ[i].minY and y < self.TAJ[i].maxY then
                        setTranslation(self.TAJ[i].index,x,y,z);
                    end;
                end;
            end;
        end;

        if self.rundumleuchtenAn then
          for i=1, self.rundumleuchtenAnz do
            rotate(self.rundumleuchten[i].rotNode, 0, dt*self.rundumleuchten[i].speed, 0);
          end;
        end;

        if InputBinding.hasEvent(InputBinding.TOGGLE_LIGHTS) then
          self.licht2 = self.licht1 and not self.licht2;
          self.licht1 = self.licht2 or  not self.licht1;
          self.lightsActive = not self.licht1;
          setVisibility(self.extra_lights.front, self.licht2);
          if self.licht2 then
            setVisibility(self.extra_lights.frontL, false);
          elseif self.licht1 then
            setVisibility(self.extra_lights.frontL, true);
          else
            setVisibility(self.extra_lights.frontL, false);
          end;
        end;
--        renderText(0.1,0.2,0.03,"RPM: "..self.motor.lastMotorRpm);
    end;

    Vehicle.update(self, dt);

    if InputBinding.hasEvent(InputBinding.TOGGLE_LIGHTS) then
      setVisibility(self.interior, not self.lightsActive);
      setVisibility(self.interiorL, self.lightsActive);
    end;
    
    for i=1, table.getn(self.attachedImplements) do
        local implement = self.attachedImplements[i];
        if implement.jointDescIndex == 1 then
            local ax, ay, az = getWorldTranslation(self.hydro.R1);
            local bx, by, bz = getWorldTranslation(self.hydro.R2);
            local x2, y2, z2 = worldDirectionToLocal(getParent(self.hydro.R2), bx-ax, by-ay, bz-az);
            setDirection(self.hydro.R2,x2,z2,-y2,0,1,0);
            local distance1 = Utils.vector3Length(ax-bx, ay-by, az-bz);
            setScale(self.hydro.R2,1,distance1,1);
            local ax, ay, az = getWorldTranslation(self.attacherJoints[1].topArm.rotationNode);
            local bx, by, bz = getWorldTranslation(implement.object.topReferenceNode);
            local distance2 = Utils.vector3Length(ax-bx, ay-by, az-bz);
            local scaleZ = distance2/self.attacherJoints[1].topArm.referenceDistance;
--            renderText(0.1,0.1,0.03,string.format("distance2: %0.3f / scaleZ: %0.3f",distance2,scaleZ));
            setTranslation(self.attacherJoints[1].topArm.translationNode,0,0,0);
            setScale(self.attacherJoints[1].topArm.translationNode,1,1,scaleZ);
        else
            local ax, ay, az = getWorldTranslation(self.hydro.F1);
            local bx, by, bz = getWorldTranslation(self.hydro.F2);
            local x1, y1, z1 = worldDirectionToLocal(getParent(self.hydro.F1), bx-ax, by-ay, bz-az);
            local x2, y2, z2 = worldDirectionToLocal(getParent(self.hydro.F2), bx-ax, by-ay, bz-az);
            setDirection(self.hydro.F1,x1,z1,-y1,0,1,0);
            setDirection(self.hydro.F2,x2,z2,-y2,0,1,0);
            local ax, ay, az = getWorldTranslation(self.attacherJoints[2].topArm.rotationNode);
            local bx, by, bz = getWorldTranslation(implement.object.topReferenceNode);
            local distance2 = Utils.vector3Length(ax-bx, ay-by, az-bz);
            local scaleZ = distance2/self.attacherJoints[2].topArm.referenceDistance;
            setTranslation(self.attacherJoints[2].topArm.translationNode,0,0,0);
            setScale(self.attacherJoints[2].topArm.translationNode,1,1,scaleZ);
        end;
    end;
end;

function agro720:draw()
    g_currentMission:addExtraPrintText("Drehzahlbegrenzer: "..string.format("%d Prozent",self.revLimit));
    if not self.showhelp then
    	g_currentMission:addExtraPrintText("Taste "..string.upper(string.char(self.keys.hilfe))..": Hilfebox einblenden");
    else
    	self.hudInfoOverlay:render();
    	local helptext={};
    	helptext[1] = "Taste "..key_convert(self.keys.rearlight)..": Arbeitslicht hinten";
    	helptext[2] = "Taste "..key_convert(self.keys.rundumleuchte)..": Rundumleuchte";
    	helptext[3] = "Taste "..key_convert(self.keys.spiegel)..": Spiegel";
    	helptext[4] = "Tasten "..key_convert(self.keys.moveAK_up).."|"..key_convert(self.keys.moveAK_dn)..": Anhngerkupplung heben/senken";
    	helptext[5] = "Tasten "..key_convert(self.keys.rev_up).."|"..key_convert(self.keys.rev_down)..": Drehzahlbegrenzer heben/senken";
    	helptext[6] = "Taste "..key_convert(self.keys.hilfe)..": Hilfebox ausblenden";
    	if self.rearlightActive then
    		 helptext[1] = helptext[1].." (an)";
    	else
    		 helptext[1] = helptext[1].." (aus)";
    	end;
    	if self.rundumleuchtenAn then
    		 helptext[2] = helptext[2].." (an)";
    	else
    		 helptext[2] = helptext[2].." (aus)";
    	end;
    	if self.drawMirror then
    		 helptext[3] = helptext[3].." (an)";
    	else
    		 helptext[3] = helptext[3].." (aus)";
    	end;
    	for i=1, table.getn(helptext) do
    		renderText((self.hudInfoPoxX+0.05),((self.hudInfoPoxY+self.hudInfoHeight) - (0.02*i)),0.02,helptext[i]);
    	end;
    end;

    Vehicle.draw(self);
end;

function agro720:onEnter()
    Utils.setEmittingState(self.exhaustParticleLow, true);
    Vehicle.onEnter(self);
end;

function agro720:onLeave()
    self.rundumleuchtenAn = false;
    for i=1, self.rundumleuchtenAnz do
       setVisibility(self.rundumleuchten[i].light, self.rundumleuchtenAn);
    end;
    self.licht1 = false;
    self.licht2 = false;
    self.rearlightActive = false;
    setVisibility(self.extra_lights.front, false);
    setVisibility(self.extra_lights.rear, self.rearlightActive);
    Utils.setEmittingState(self.exhaustParticleLow, false);

    Vehicle.onLeave(self);
end;

function key_convert(key)
    if key ~= nil then 
	    local keyname = "";
	    if key > 32 and key < 123 then
	    	keyname = string.upper(string.char(key));
	    elseif key > 255 and key < 266 then
	    	keyname = "NUM "..string.char(key-208);
	    else 
	    	keyname = "";
	    end;
	    return(keyname);
	  end;
end;
